package holmos.webtest.junitextentions;

import holmos.webtest.basetools.HolmosBaseTools;
import holmos.webtest.junitextentions.annotations.MultiThread;
import holmos.webtest.junitextentions.annotations.RunAfter;
import holmos.webtest.junitextentions.annotations.RunCount;
import holmos.webtest.log.MyLogger;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.junit.runner.Description;
import org.junit.runner.manipulation.Filter;
import org.junit.runner.manipulation.NoTestsRemainException;
import org.junit.runner.manipulation.Sorter;
import org.junit.runner.notification.RunNotifier;
import org.junit.runners.BlockJUnit4ClassRunner;
import org.junit.runners.model.FrameworkMethod;
import org.junit.runners.model.InitializationError;
import org.junit.runners.model.RunnerScheduler;
import org.junit.runners.model.Statement;

/**
 * junit的case扩展顺序执行器
 * @param <T>
 * */
public class HolmosRunner extends BlockJUnit4ClassRunner{
	private static MyLogger logger=MyLogger.getLogger(HolmosRunner.class);
	private int threadCount=0;
	private Filter fFilter= null;
	private RunnerScheduler fScheduler= new RunnerScheduler() {	
		public void schedule(Runnable childStatement) {
			if(childStatement instanceof Thread)
				((Thread) childStatement).start();
			else
				childStatement.run();
		}
		public void finished() {
			// do nothing
		}
	};

	private Sorter fSorter= null;
	{
		fSorter=new Sorter(new Comparator<Description>() {
			@Override
			public int compare(Description o1, Description o2) {
				if(o1.getAnnotation(RunAfter.class)==null)return 0;
				String priorMethodOfO1=o1.getAnnotation(RunAfter.class).mothodName();
				if(priorMethodOfO1.equalsIgnoreCase(o2.getMethodName())){
					return 1;
				}
				return 0;
			}
		});
	}

	public HolmosRunner(Class<?> klass) throws InitializationError {
		super(klass);
		this.sort(fSorter);
	}
	/**
	 * Returns a {@link Statement}: Call {@link #runChild(Object, RunNotifier)}
	 * on each object returned by {@link #getChildren()} (subject to any imposed
	 * filter and sort)
	 */
	@Override
	protected Statement childrenInvoker(final RunNotifier notifier) {
		return new Statement() {
			@Override
			public void evaluate() {
				runHolmosChildren(notifier);
			}
		};
	}
	

	private List<FrameworkMethod> getFilteredChildren() {
		ArrayList<FrameworkMethod> filtered= new ArrayList<FrameworkMethod>();
		for (FrameworkMethod each : getChildren())
			if (shouldRun(each))
				try {
					filterChild(each);
					sortChild(each);
					filtered.add(each);
				} catch (NoTestsRemainException e) {
					// don't add it
				}
		Collections.sort(filtered, comparator());
		return filtered;
	}
	private void runHolmosChildren(final RunNotifier notifier) {
		for (final FrameworkMethod each : getFilteredChildren()){
			Runnable excute=null;
			Runnable excuteFeed=new Runnable() {			
				public void run() {
					int repeatCount=1;
					if(null!=each.getAnnotation(RunCount.class))
						repeatCount=each.getAnnotation(RunCount.class).count();
					for(int i=0;i<repeatCount;i++){
						threadCount++;
						runChild(each, notifier);
						threadCount--;
					}
				}
			};
			if(getDescription().getAnnotation(MultiThread.class)!=null)
				 excute=new Thread(excuteFeed);
			else if(each.getAnnotation(MultiThread.class)!=null)
				 excute=new Thread(excuteFeed);
			else
				 excute=excuteFeed;
			fScheduler.schedule(excute);
		}
		//等待他们最后一个运行完，finish
		while(threadCount!=0){
			HolmosBaseTools.sleep(50);
		}
		fScheduler.finished();
	}
	
	private boolean shouldRun(FrameworkMethod each) {
		return fFilter == null || fFilter.shouldRun(describeChild(each));
	}
	private void filterChild(FrameworkMethod child) throws NoTestsRemainException {
		if (fFilter != null)
			fFilter.apply(child);
	}
	private void sortChild(FrameworkMethod child) {
		fSorter.apply(child);
	}
	private Comparator<? super FrameworkMethod> comparator() {
		return new Comparator<FrameworkMethod>() {
			public int compare(FrameworkMethod o1, FrameworkMethod o2) {
				return fSorter.compare(describeChild(o1), describeChild(o2));
			}
		};
	}
}
